import matplotlib.pyplot as plt
from neupy import algorithms, layers, environment, estimators
import numpy as np
import openbabel
import pandas as pd
import pubchempy as pcp
import pybel
from pybel import Smarts, readstring
from sklearn import linear_model, metrics
from sklearn.cross_decomposition import PLSRegression
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from sklearn.linear_model import Ridge
from sklearn.metrics import mean_squared_error, r2_score
from sklearn.model_selection import cross_val_score, GridSearchCV, RandomizedSearchCV, ShuffleSplit, train_test_split
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import MinMaxScaler, PolynomialFeatures
from sklearn.svm import LinearSVC
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neural_network import MLPRegressor


def Database():
    """compile data sets into a data library, the output will be a DataFrame"""
    data_1 = pd.read_excel('data/Flash Point and Cetane Number Predictions for Fuel Compounds.xls', skiprows=3) ###load data
    data = data_1[['Name','Family', 'FP Exp.', 'CN Exp.']]  ###select columns
    result_1 = data.drop(index=0)   ###rearrange the index
    result_1.reset_index(drop=True, inplace=True)
    names = ['Name','Family', 'Flash Point', 'Cetane Number']   ###rename columns
    result_1.columns = names
    data_2 = pd.read_excel('data/Flash Point and Cetane Number Predictions for Fuel Compounds.xls', skiprows=4)
    result_2 = data_2.loc[: , '-H': 'aaCa']   ###select specific columns
    smarts = ["[H]", "[CX4H3]", "[CX4H2]", "[CX4H1]", "[CX4H0]", "[CX3H2]", "[CX3H1]", "[CX3H0]", "[CX2H1]","[CX2H0]", 
"[CX4H2R]", "[CX4H1R]", "[CX4H0R]","[CX3H1R]","[CX3H0R]","[cX3H1](:*):*", "[cX3H0](:*)(:*)*", "[OX2H1]", "[OX2H1][cX3]:[c]", 
"[OX2H0]", "[OX2H0R]", "[oX2H0](:*):*", "[CX3H0]=[O]", "[CX3H0R]=[O]", "[CX3H1]=[O]", "[CX3H0](=[O])[OX2H1]", 
"[CX3H0](=[O])[OX2H0]", "[cX3H0](:*)(:*):*"]   ###rename functional groups to SMARTS
    result_2.columns = smarts
    result = pd.concat([result_1, result_2], axis=1)   ###combine two dataframes into one dataframe
    return result


def SMILES(CID):
    """
    Description:
    This function produces a SMILES from CID number input 
    """
    #CID number can be gained from URL:https://pubchem.ncbi.nlm.nih.gov/
    c = pcp.Compound.from_cid(CID)
    SMILES = c.canonical_smiles  ###create SMILES structure
    return SMILES


def descriptor_generator(CID):
    """Generate the number of each functional group for specific compound according to input CID"""
    fg = ["[H]", "[CX4H3]", "[CX4H2]", "[CX4H1]", "[CX4H0]", "[CX3H2]", "[CX3H1]", "[CX3H0]", "[CX2H1]", 
              "[CX2H0]", "[CX4H2R]", "[CX4H1R]", "[CX4H0R]","[CX3H1R]","[CX3H0R]","[cX3H1](:*):*", 
              "[cX3H0](:*)(:*)*", "[OX2H1]", "[OX2H1][cX3]:[c]", "[OX2H0]", "[OX2H0R]", "[oX2H0](:*):*", 
              "[CX3H0]=[O]", "[CX3H0R]=[O]", "[CX3H1]=[O]", "[CX3H0](=[O])[OX2H1]", "[CX3H0](=[O])[OX2H0]", 
              "[cX3H0](:*)(:*):*"]  ###define SMARTS of functional groups list
    counts = []   ###count functional groups
    result = SMILES(CID)  ###generate SMILES of compound
    mol = readstring("smi", result)   ###load SMILES
    for i in range(len(fg)):
        smarts = Smarts(fg[i])   ###load SMARTS
        n = smarts.findall(mol)  ###find specific functional group, return will be tuples in a list
        counts.append(len(n))    ###record number of functional group
    X = pd.DataFrame(np.array(counts).reshape(1, -1))  ###reshape the counts
    return X


def df_prediction(family, prop):
    """
    This function is used to create train and test data for prediction.
    """
    test_size = 0.1   ###define test size 
    data = Database()   ###load, select, clear NaN data
    data_f = data[data.Family == family]
    df = data_f[np.isfinite(data_f[prop])]
    if df.shape == (0, 32):
        raise Exception ('shape', 'We are short of data!')
        return
    else:
        train, test = train_test_split(df, test_size=test_size, random_state=17)  ###split data
        return train, test


def plot(model, prop, family):
    """
    This function is used to make parity plot, mse vs. bootstrap samples, r_2 vs. bootstrap samples.
    """
    iteration = 10  ###define iteration for bootstrap samples
    
    train, test = df_prediction(family, prop)
    X_train = train[train.columns[4:]]  ###select functional groups
    X_test = test[test.columns[4:]]
    n = np.arange(1, iteration+1)       ###set bootstrap sample
    mses = []    ###place mse
    r2s = []     ###place r2
    
    fig = plt.figure(figsize=(18, 6))   ###adjust the figsize
    plt.subplot(131)                    ###make parity plot
    plt.scatter(train[prop], model.predict(X_train), label='training data')
    plt.scatter(test[prop], model.predict(X_test), color='r', label='testing data')
    plt.xlabel(prop, fontsize=16)
    plt.ylabel(prop, fontsize=16)
    plt.legend(fontsize=16)
    plt.title('Parity Plot', fontsize=16)
    
    for i in n:
        mse, r2 = bootstrap(prop, i, family, model)  ###get mse and r2 average for different samples
        mses.append(mse)
        r2s.append(r2)
    
    plt.subplot(132)    ###make mse vs. bootstrap samples
    plt.plot(n, mses)
    plt.xlabel('Number of Bootstrap Samples', fontsize=16)
    plt.ylabel('Mean Squared Error (MSE)', fontsize=16)
    plt.title('Number of Bootstrap Samples vs. MSE', fontsize=16)
    
    plt.subplot(133)    ###make r2 vs. bootstrap samples
    plt.plot(n, r2s)
    plt.xlabel('Number of Bootstrap Samples', fontsize=16)
    plt.ylabel('$R^2$', fontsize=16)
    plt.title('Number of Bootstrap Samples vs. $R^2$', fontsize=16)
    return fig


def bootstrap(prop, iteration, family, model):
    """
    This function is used to validate model by resampling method, it will generate mse and r2 average.
    """
    fraction = 0.8  ###define resample fraction
    data = Database()  ###load dataframe
    data_f = data[data.Family == family]  ###select specific family to resample
    df = data_f[np.isfinite(data_f[prop])]  ###clear the data is NaN
    mse = []   ###place mse values
    r2 = []    ###place r2 values
    for i in range(iteration):
        sample = df.sample(frac=fraction, replace=True)   ###sample from data
        X = sample[sample.columns[4:]]  ###select functional groups
        y = sample[prop]     ###select properties
        y_pred = model.predict(X)   ###build model and predict
        mse.append(mean_squared_error(y, y_pred))   ###compute mse
        r2.append(r2_score(y, y_pred))   ###compute r2
    mse_avg = np.mean(mse)    ###average mse
    r2_avg = np.mean(r2)      ###average r2
    return mse_avg, r2_avg


def OLS_train(family, prop):
    """
    This function is used to train model according to Ordinary Least Squares(linear model). 
    """
    train, test = df_prediction(family, prop)  ###create data for train and test
    OLS = linear_model.LinearRegression()   ###build model
    train_X = train[train.columns[4:]]   ###select functional groups
    OLS.fit(train_X, train[prop])    ###train model
    return OLS     ###return model

def OLS_test(family, prop):
    """
    This function is used to test and make plots according to OLS model.
    """
    model= OLS_train(family, prop)
    fig = plot(model, prop, family)  ###make plots
    return fig

def OLS_pred(family, prop, fg):
    """
    This function is used to predict properties according to OLS model.
    """
    model= OLS_train(family, prop)
    result = model.predict(fg)[0]
    return result


def PLS_train(family, prop):
    """
    This function is used to train model according to Partial Least Squares(linear model).
    """
    train, test = df_prediction(family, prop)  ###create data for train and test
    PLS = PLSRegression()          ###build model
    train_X = train[train.columns[4:]]   ###select functional groups
    PLS.fit(train_X, train[prop])   ###train model
    return PLS     ###return model

def PLS_test(family, prop):
    """
    This function is used to test and make plots according to PLS model.
    """
    model = PLS_train(family, prop)
    fig = plot(model, prop, family)  ###make plots
    return fig

def PLS_pred(family, prop, fg):
    """
    This function is used to predict properties according to PLS model.
    """
    model = PLS_train(family, prop)
    result = model.predict(fg)[0][0]
    return result


def PNR_train(family, prop):
    """
    This function is used to train model according to Polynomial Regression(nonlinear model). 
    """
    train, test = df_prediction(family, prop)    ###create data for train and test
    PNR = make_pipeline(PolynomialFeatures(), Ridge())   ###build model
    train_X = train[train.columns[4:]]   ###select functional groups
    PNR.fit(train_X, train[prop])    ###train model
    return PNR         ###return model

def PNR_test(family, prop):
    """
    This function is used to test and make plots according to PNR model.
    """
    model = PNR_train(family, prop)
    fig = plot(model, prop, family)  ###make plots
    return fig

def PNR_pred(family, prop, fg):
    """
    This function is used to predict properties according to PNR model.
    """
    model = PNR_train(family, prop)
    result = model.predict(fg)[0]
    return result


def GRNN_train(family, prop):
    """This function is used to predict properties by using the General Regression Neural Network model."""
    train, test = df_prediction(family, prop)  ###create data for train and test
    x_train = train[train.columns[4:]]   ###select functional groups
    y_train = train[prop]    ###select prop groups

    scaler = MinMaxScaler(feature_range=(0, 1))  #Rescale model
    rescaledX = scaler.fit_transform(x_train)   #Rescale x
    np.set_printoptions(precision=4) # summarize transformed data for x,, and also set up the descimal place of the value
    
    grnn = algorithms.GRNN(std=0.4,verbose=False,)    #Set up the model
    grnn.train(x_train, y_train)  #Train the model
    return grnn

def GRNN_test(family, prop):
    """
    This function is used to make plots according to GRNN model.
    """
    model= GRNN_train(family, prop)
    fig = plot(model, prop, family)  ###make plots
    return fig

def GRNN_pred(family, prop, fg):
    """
    This function is used to predict properties according to GRNN model.
    """
    model = GRNN_train(family, prop)
    result = model.predict(fg)[0][0]
    return result


def MLPR_train(family, prop):
    """This function is used to predict properties by using the Multiple Layers Perception Regression model."""
    # Input data and define the parameters
    data = Database()
    data_f = data[data.Family == family]
    df = data_f[np.isfinite(data_f[prop])]
    x = df.loc[:,'[H]':'[cX3H0](:*)(:*):*']
    y = df[prop]
    
    array_x = x.values
    array_y = y.values
    
    scaler = MinMaxScaler(feature_range=(0, 1))   #Rescale model
    rescaledX = scaler.fit_transform(array_x)   #Rescale x
    np.set_printoptions(precision=4) # summarize transformed data for x,, and also set up the descimal place of the value
    
    x_train, x_test, y_train, y_test = train_test_split(rescaledX, array_y, test_size=0.1, random_state=17)
        
    mlpr = MLPRegressor(hidden_layer_sizes=(1000,),activation='identity', solver='sgd', learning_rate='adaptive', 
max_iter=4000, verbose=False)   #Set up the model
    mlpr.fit(x_train, y_train)   #Train the model
    return mlpr, x_train, x_test, y_train, y_test

def MLPR_test(family, prop):
    """
    This function is used to make plots according to MLPR model.
    """
    model, x_train, x_test, y_train, y_test = MLPR_train(family, prop)
    y_predict = model.predict(x_test)
    y_predict_train = model.predict(x_train)
    
    fig = plt.figure(figsize=(18, 6))
    plt.scatter(y_test, y_predict, color='r', label='testing data')
    plt.scatter(y_train, y_predict_train,  label='training data')
    plt.xlabel(prop+'_Actual', fontsize=16)
    plt.ylabel(prop+'_Predict', fontsize=16)
    plt.title('Parity Plot', fontsize=16)
    plt.legend()
    return fig

def MLPR_pred(family, prop, fg):
    """
    This function is used to predict properties according to MLPR model.
    """
    model = MLPR_train(family, prop)[0]
    result = model.predict(fg)[0]
    return result


def data_clean():
    """split data and generate train & test subset"""
    df = Database()
    train, test = train_test_split(df, test_size=0.1, random_state=2)
    a = train.loc[:,'[H]': '[cX3H0](:*)(:*):*']
    X_train = a.mask(a>0, 1)# transfer the descriptor numbers into 1/0
    y_train = train['Family']

    b = test.loc[:,'[H]': '[cX3H0](:*)(:*):*']
    X_test = b.mask(b>0, 1)
    y_test = test['Family']
    return X_train, y_train, X_test, y_test


def train_knn(k, X_train, y_train):
    """use knn method to train data"""
    knn = KNeighborsClassifier(n_neighbors=k)
    knn.fit(X_train, y_train)
    return knn

def test_knn():
    """give classification accuracy"""
    X_train, y_train, X_test, y_test = data_clean()
    k = 5 # fix k at 5
    knn = train_knn(k, X_train, y_train)
    test_pred = knn.predict(X_test)
    train_pred = knn.predict(X_train)
    acc = metrics.accuracy_score(y_test, test_pred)
    print('k =', k)
    print('Accuracy =', acc)
    return acc
    
def predict_family_knn(X):
    """predit family of import molecule X"""
    X_train, y_train, X_test, y_test = data_clean()
    k = 5 # fix k at 5
    knn = train_knn(k, X_train, y_train)
    y_pred = knn.predict(X)
    return y_pred

def plot_knn(y_pred):
    """plot a y_pred vs. y_actual figure and show the prediction on it"""
    X_train, y_train, X_test, y_test = data_clean()
    k = 5 # fix k at 5
    knn = train_knn(k, X_train, y_train)
    test_pred = knn.predict(X_test)
    train_pred = knn.predict(X_train)
    acc = metrics.accuracy_score(y_test, test_pred)
    ax = plt.figure(figsize=(9,4))
    plt.plot([0,7], [0,7], color='k')
    plt.scatter(y_train, train_pred, marker='s', s=100,c='c', label='train')
    plt.scatter(y_test, test_pred, marker='d', s=100, c='orange', label='test')
    plt.scatter(y_pred, y_pred, marker='*', s=100, c='r', label='prediction') # plot the predictoin dot
    plt.xticks(rotation='40')
    plt.xlabel('Actual Family', fontsize=15)
    plt.ylabel('Predicted Family', fontsize=15)
    plt.title('Knn Classification (k=%d, Accuracy=%.5f)' % (k, acc),fontsize=20)
    plt.legend()
    return ax


def train_lda(X_train, y_train):
    """use lda method to train data"""
    lda = LinearDiscriminantAnalysis()
    lda.fit(X_train, y_train)
    return lda

def test_lda():
    """give classification accuracy"""
    X_train, y_train, X_test, y_test = data_clean()
    lda = train_lda(X_train, y_train)
    test_pred = lda.predict(X_test)
    acc = metrics.accuracy_score(y_test, test_pred)
    print('Accuracy = ', acc)
    return acc

def predict_family_lda(X):
    """predit family of import molecule X"""
    X_train, y_train, X_test, y_test = data_clean()
    lda = train_lda(X_train, y_train)
    y_pred = lda.predict(X)
    return y_pred

def plot_lda(y_pred):
    """plot a y_pred vs. y_actual figure and show the prediction on it"""
    X_train, y_train, X_test, y_test = data_clean()
    lda = train_lda(X_train, y_train)
    test_pred = lda.predict(X_test)
    train_pred = lda.predict(X_train)
    acc = metrics.accuracy_score(y_test, test_pred)
    ax = plt.figure(figsize=(9,4))
    plt.plot([0,7], [0,7], color='k')
    plt.scatter(y_train, train_pred, marker='s', s=100,c='c', label='train')
    plt.scatter(y_test, test_pred, marker='d', s=100, c='orange', label='test')
    plt.scatter(y_pred, y_pred, marker='*', s=100, c='r', label='prediction')# plot the predictoin dot
    plt.xticks(rotation='40')
    plt.xlabel('Actual Family', fontsize=15)
    plt.ylabel('Predicted Family', fontsize=15)
    plt.title('LDA Classification (Accuracy=%.5f)' % (acc),fontsize=20)
    plt.legend()
    return ax


def train_svm(X_train, y_train):
    """use lda method to train data"""
    svm = LinearSVC(random_state=0)
    svm.fit(X_train, y_train)
    return svm

def test_svm():
    """give classification accuracy"""
    X_train, y_train, X_test, y_test = data_clean()
    svm = train_svm(X_train, y_train)
    test_pred = svm.predict(X_test)
    acc = metrics.accuracy_score(y_test, test_pred)
    print('Accuracy = ', acc)
    return acc

def predict_family_svm(X):
    """predit family of import molecule X"""
    X_train, y_train, X_test, y_test = data_clean()
    svm = train_svm(X_train, y_train)
    y_pred = svm.predict(X)
    return y_pred

def plot_svm(y_pred):
    """plot a y_pred vs. y_actual figure and show the prediction on it"""
    X_train, y_train, X_test, y_test = data_clean()
    svm = train_svm(X_train, y_train)
    test_pred = svm.predict(X_test)
    train_pred = svm.predict(X_train)
    acc = metrics.accuracy_score(y_test, test_pred)
    ax = plt.figure(figsize=(9,4))
    plt.plot([0,7], [0,7], color='k')
    plt.scatter(y_train, train_pred, marker='s', s=100,c='c', label='train')
    plt.scatter(y_test, test_pred, marker='d', s=100, c='orange', label='test')
    plt.scatter(y_pred, y_pred, marker='*', s=100, c='r', label='prediction')# plot the predictoin dot
    plt.xticks(rotation='40')
    plt.xlabel('Actual Family', fontsize=15)
    plt.ylabel('Predicted Family', fontsize=15)
    plt.title('SVM Classification (Accuracy=%.5f)' % (acc),fontsize=20)
    plt.legend()
    return ax




import sys
sys.path.append('../..')

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import LinearSVC

from MyProject import core


def test_Database():
    '''Check the shape of the database'''
    result = core.Database()
    assert result.shape == (1159, 32), "The database is not right"  ##test the shape of the database
    return

def test_SMILES():
    """Check the SMILES function could generate the correct molecular structure,based on input CID number"""
    result1 = core.SMILES(175854)  ##check SMILES structures
    result2 = core.SMILES(13526104)
    result3 = core.SMILES(887)
    
    assert result1 == 'C(C=CCO)O', 'The chemcial (2E)-but-2-ene-1,4-diol converts wrong!'
    assert result2 == 'CCCCC(CC)(CO)C(C)O', 'The chemcial 2-Butyl-2-ethylbutane-1,3-diol converts wrong!'
    assert result3 == 'CO', 'The chemcial methanol converts wrong'
    return

def test_descriptor_generator():
    """Check descriptor_generator can generate correct numbers of functional groups for 
    specific compound according to CID"""
    result1 = core.descriptor_generator(887).values.T.tolist()  ##test methane
    result2 = core.descriptor_generator(175854).values.T.tolist()   ##test 2-Butene-1,4-diol, (2E)-
    result3 = core.descriptor_generator(61038).values.T.tolist()   ##test 2-Ethyl-2-butyl-1,3-propanediol

    assert result1 == [[4],[1],[0],[0],[0], [0], [0],[0],[0],[0], [0],[0], [0], [0],[0], [0],[0], [1], [0],[0], [0], [0], [0], [0], [0], [0], [0], [0]], (
                      "The descriptor_generator does not work properly")
    assert result2 == [[8], [0], [2], [0],[ 0],[ 0], [2], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0], [2], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0]], (
                      "The descriptor_generator does not work properly")
    assert result3 == [[20], [2], [6], [0], [1], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0], [2], [0], [0], [0], [0], [0], [0], [0], [0], [0], [0]], (
                      "The descriptor_generator does not work properly")
    return

def test_df_prediction():
    """
    This function is used to test df_prediction.
    """
    family = 'Alcohols'   ###define parameters
    prop = 'Flash Point'
    train = core.df_prediction(family, prop)[0]
    test = core.df_prediction(family, prop)[1]
    assert train.shape == (60, 32), "The training data are not right"  ###test the shape of training data
    assert test.shape == (7, 32), "The testing data are not right"    ### test the shape of testing data
    return

def test_bootstrap():
    """Check the type of outputs are correct"""
    result1 = type(core.bootstrap('Flash Point', 10, 'Esters', core.OLS_train('Esters', 'Flash Point')))
    result2 = type(core.bootstrap('Flash Point', 10, 'Esters', core.OLS_train('Esters', 'Flash Point'))[0])
    assert result1 == tuple, 'The output is wrong'
    assert result2 == np.float64, 'The output is wrong'
    return

def test_plot():
    """Check the type of outputs are correct"""
    assert type(core.plot(core.OLS_train('Esters','Cetane Number'), 'Cetane Number', 'Esters')) == matplotlib.figure.Figure, 'The output is wrong :('
    return

def test_model():
    result1 = type(core.OLS_pred('Esters', 'Flash Point', core.descriptor_generator(887)))
    result2 = type(core.OLS_pred('Esters', 'Cetane Number', core.descriptor_generator(65115)))
    assert result1 == np.float64, 'The output is wrong'
    assert result2 == np.float64, 'The output is wrong'
    return

X_train, y_train, X_test, y_test = core.data_clean()
X=X_train.iloc[0:1,:]
y_pred = 'Esters'

def test_data_clean():
    # four results of data sets should be provided
    assert len(core.data_clean()) == 4, "clean_data() breaks"
    assert type(core.data_clean()) == tuple, "clean_data() breaks"
    return 

def test_train_knn():
    k = 5
    knn = core.train_knn(k, X_train, y_train)
    # knn should be a classifier
    type(knn) == KNeighborsClassifier, "train_knn() breaks"
    return 

def test_test_knn():
    k = 5
    # accuarcy should be possitive
    assert core.test_knn() > 0, "test_knn() breaks"
    return
    

def test_predict_family_knn():
    # prediction should be an array of family
    assert type(core.predict_family_knn(X)) == np.ndarray, "predict_family_knn() breaks"
    return

def test_plot_knn():
    # this fuction should provide a figure
    assert type(core.plot_knn(y_pred)) == plt.Figure, "plot_knn() breaks"
    return 

def test_train_lda():
    # lda should be a classifier
    lda = core.train_lda(X_train, y_train)
    type(lda) == LinearDiscriminantAnalysis, "train_lda() breaks"
    return 

def test_test_lda():
    # accuarcy should be possitive
    assert core.test_lda() > 0, "test_lda() breaks"
    return
    

def test_predict_family_lda():
    # prediction should be an array of family
    assert type(core.predict_family_lda(X)) == np.ndarray, "predict_family_lda() breaks"
    return

def test_plot_lda():
    # this fuction should provide a figure
    assert type(core.plot_lda(y_pred)) == plt.Figure, "plot_lda() breaks"
    return

def test_train_svm():
    # svm should be a classifier
    svm = core.train_svm(X_train, y_train)
    type(svm) == LinearSVC, "train_svm() breaks"
    return 

def test_test_svm():
    # accuarcy should be possitive
    assert core.test_svm() > 0, "test_svm() breaks"
    return
    
def test_predict_family_svm():
    # prediction should be an array of family
    assert type(core.predict_family_svm(X)) == np.ndarray, "predict_family_svm() breaks"
    return

def test_plot_svm():
    # this fuction should provide a figure
    assert type(core.plot_svm(y_pred)) == plt.Figure, "plot_svm() breaks"
    return 



#import matplotlib setting
import matplotlib
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg
from matplotlib.figure import Figure
matplotlib.use("TkAgg")

#import tkinter
from tkinter import *
import tkinter as tk
import tkinter.messagebox as tkMessageBox

#import all related functions
from core import *

#Basic frame that incorporate settings for different frames
class QSPR_Modeling(tk.Tk):

    def __init__(self,*args,**kwargs):
        tk.Tk.__init__(self,*args,**kwargs)
        tk.Tk.wm_title(self,"QSPR Modeling")

        container=tk.Frame(self,width=300,height=200,bg="purple")
        container.pack(side=tk.TOP,fill=tk.BOTH,expand=True)
        container.grid_rowconfigure(0,weight=1)
        container.grid_columnconfigure(0,weight=1)

        self.frames={}
        for F in (Model_Selection,Models,Graphs):
            frame = F(container,self)
            self.frames[F]= frame
            frame.grid(row=0,column=0,sticky="nsew") #assign the position
        self.show_frame(Model_Selection)

    def show_frame(self,cont):
        frame=self.frames[cont]
        frame.tkraise()

#Main frame on the topmost
class Model_Selection(tk.Frame):

    def quit(self):
        msg=tkMessageBox.askyesno("Program: ","Are you sure to quit? ")
        if msg:
            exit()

    def ref(self):
        tkMessageBox.showinfo("Reference: ","[1].Saldana, D. A.; Starck, L.; Mougin,center P.; Rousseau, B.; Pidol, L.;"
                                            " Jeuland, N.; Creton, B. Energy & Fuels 2011, 25 (9)")

    def acknow(self):
        tkMessageBox.showinfo("Acknowledgements: ","This software is supported by DIRECT program in University of Washington CEI Group. "
                                                   "Designers include: Jingtian Zhang, Renlong Zheng, Cheng Zeng, Chenggang Xi")

    def intro(self):
        tkMessageBox.showinfo("Introduction:","The main goal of this software was to predict flash points "
                                              "and cetane number of an known biofuel. The user can input CID number"
                                              "and obtain the predicted values gained by machine learning alogrithms built"
                                              "in this software. Classification and linear regression were used to train and test models.")

    def get_CID(self, txtCID):
        global CID
        CID = int(txtCID.get())
        if CID in range(1, 1000000000):
            tkMessageBox.showinfo("CID", CID)
            return CID
        else:
            tkMessageBox.askretrycancel('CID: ',"please check your CID number make sure the input stays within the range defined")

    def __init__(self,parent,controller):
        #CID should be a global variable in order to go through every step of QSPR modeling
        '''This class configures and populates the toplevel window.
                   top is the toplevel containing window.'''
        _bgcolor = '#d9d9d9'  # X11 color: 'gray85'
        _fgcolor = '#000000'  # X11 color: 'black'
        _compcolor = '#d9d9d9'  # X11 color: 'gray85'
        _ana1color = '#d9d9d9'  # X11 color: 'gray85'
        _ana2color = '#d9d9d9'  # X11 color: 'gray85'
        font11 = "-family {Segoe UI Black} -size 12 -weight bold " \
                 "-slant roman -underline 0 -overstrike 0"
        font12 = "-family {Segoe UI Black} -size 11 -weight bold " \
                 "-slant roman -underline 0 -overstrike 0"
        font13 = "-family {Segoe UI Black} -size 11 -weight bold " \
                 "-slant roman -underline 1 -overstrike 0"
        font14 = "-family {Courier New} -size 12 -weight normal -slant" \
                 " roman -underline 0 -overstrike 0"
        font9 = "-family {Segoe UI} -size 9 -weight normal -slant " \
                "roman -underline 0 -overstrike 0"

        tk.Frame.__init__(self, parent)

        btncancel=tk.Button(self)
        btncancel.place(relx=0.42, rely=0.6, height=73, width=166)
        btncancel.configure(activebackground="#c0c0c0")
        btncancel.configure(activeforeground="#000000")
        btncancel.configure(background="#4f24a6")
        btncancel.configure(disabledforeground="#a3a3a3")
        btncancel.configure(font=font12)
        btncancel.configure(foreground="#ffffff")
        btncancel.configure(highlightbackground="#d9d9d9")
        btncancel.configure(highlightcolor="black")
        btncancel.configure(pady="0")
        btncancel.configure(command=self.quit)
        btncancel.configure(text='''Cancel''')

        btnModelSelection = Button(self)
        btnModelSelection.place(relx=0.6, rely=0.6, height=73, width=170)
        btnModelSelection.configure(activebackground="#d9d9d9")
        btnModelSelection.configure(activeforeground="#000000")
        btnModelSelection.configure(background="#4f24a6")
        btnModelSelection.configure(disabledforeground="#a3a3a3")
        btnModelSelection.configure(font=font12)
        btnModelSelection.configure(foreground="#ffffff")
        btnModelSelection.configure(highlightbackground="#d9d9d9")
        btnModelSelection.configure(highlightcolor="black")
        btnModelSelection.configure(pady="0")
        btnModelSelection.configure(command=lambda:controller.show_frame(Models))
        btnModelSelection.configure(text='''Model Selection''')

        txtCID = Entry(self)
        txtCID.place(x=580, y=200, height=45, width=194)
        txtCID.configure(background="white")
        txtCID.configure(disabledforeground="#a3a3a3")
        txtCID.configure(font=font14)
        txtCID.configure(foreground="#000000")
        txtCID.configure(highlightbackground="#d9d9d9")
        txtCID.configure(highlightcolor="black")
        txtCID.configure(insertbackground="black")
        txtCID.configure(justify=CENTER)
        txtCID.configure(selectbackground="#c4c4c4")
        txtCID.configure(selectforeground="black")

        labelCID = Label(self)
        labelCID.place(relx=0.29, rely=0.25, height=45, width=132)
        labelCID.configure(activebackground="#f9f9f9")
        labelCID.configure(activeforeground="black")
        labelCID.configure(background="#b197e8")
        labelCID.configure(disabledforeground="#a3a3a3")
        labelCID.configure(font=font11)
        labelCID.configure(foreground="#000000")
        labelCID.configure(highlightbackground="#d9d9d9")
        labelCID.configure(highlightcolor="black")
        labelCID.configure(text='''CID''')

        mes59 = Message(self)
        mes59.place(x=410, y=20, height=160, width=536)
        mes59.configure(background="#b197e8")
        mes59.configure(font=font13)
        mes59.configure(foreground="#800080")
        mes59.configure(highlightbackground="#d9d9d9")
        mes59.configure(highlightcolor="black")
        mes59.configure(justify=CENTER)
        mes59.configure(text='''This software is designed to predict flash points of different organics by applying machine learning alogrithms.
        Linear Regression Methods include: GRNN, OLS, PLS, MLPR, KNN and NLR. Classifiers include: KNN''')
        mes59.configure(width=536)

        btnacknow = Button(self)
        btnacknow.place(relx=0.79, rely=0.70, height=73, width=196)
        btnacknow.configure(activebackground="#d9d9d9")
        btnacknow.configure(activeforeground="#000000")
        btnacknow.configure(background="#4f24a6")
        btnacknow.configure(disabledforeground="#a3a3a3")
        btnacknow.configure(font=font12)
        btnacknow.configure(foreground="#ffffff")
        btnacknow.configure(highlightbackground="#d9d9d9")
        btnacknow.configure(highlightcolor="#ffffff")
        btnacknow.configure(pady="0")
        btnacknow.configure(command=self.acknow)
        btnacknow.configure(text='''Acknowledgements''')

        btnref = Button(self)
        btnref.place(relx=0.42, rely=0.80, height=73, width=166)
        btnref.configure(activebackground="#d9d9d9")
        btnref.configure(activeforeground="#4f24a6")
        btnref.configure(background="#5025a7")
        btnref.configure(disabledforeground="#a3a3a3")
        btnref.configure(font=font12)
        btnref.configure(foreground="#ffffff")
        btnref.configure(highlightbackground="#d9d9d9")
        btnref.configure(highlightcolor="black")
        btnref.configure(pady="0")
        btnref.configure(command=self.ref)
        btnref.configure(text='''Reference''')

        btnIntro = Button(self)
        btnIntro.place(relx=0.6, rely=0.80, height=73, width=166)
        btnIntro.configure(activebackground="#d9d9d9")
        btnIntro.configure(activeforeground="#000000")
        btnIntro.configure(background="#4f24a6")
        btnIntro.configure(disabledforeground="#a3a3a3")
        btnIntro.configure(font=font12)
        btnIntro.configure(foreground="#ffffff")
        btnIntro.configure(highlightbackground="#d9d9d9")
        btnIntro.configure(highlightcolor="black")
        btnIntro.configure(pady="0")
        btnIntro.configure(command=lambda: self.get_CID(txtCID))
        btnIntro.configure(text='''Get CID''')

        background_image = tk.PhotoImage(file="./2.png")
        label_image = Label(self, image=background_image)
        label_image.image = background_image
        label_image.place(relx=0.0, rely=0.0, height=200, width=200)


#Select models and properties
class Models(Model_Selection):

    def __init__(self,parent,controller):
        '''This class configures and populates the toplevel window.
                   top is the toplevel containing window.'''
        _bgcolor = '#d9d9d9'  # X11 color: 'gray85'
        _fgcolor = '#000000'  # X11 color: 'black'
        _compcolor = '#d9d9d9'  # X11 color: 'gray85'
        _ana1color = '#d9d9d9'  # X11 color: 'gray85'
        _ana2color = '#d9d9d9'  # X11 color: 'gray85'
        font10 = "-family {Courier New} -size 10 -weight normal -slant" \
                 " roman -underline 0 -overstrike 0"
        font12 = "-family {Segoe UI Black} -size 11 -weight bold " \
                 "-slant italic -underline 0 -overstrike 0"
        font13 = "-family {Segoe UI} -size 11 -weight normal -slant " \
                 "roman -underline 0 -overstrike 0"

        tk.Frame.__init__(self,parent)
        lab42 = LabelFrame(self)
        lab42.place(x=30, y=20, height=395, width=180)
        lab42.configure(relief=RAISED)
        lab42.configure(font=font12)
        lab42.configure(foreground="#000000")
        lab42.configure(relief=RAISED)
        lab42.configure(text='''Numerical Regression''')
        lab42.configure(background="#c0c0c0")
        lab42.configure(width=180)

        lab42_lis46 = Listbox(lab42, exportselection=0)
        lab42_lis46.place(x=40, y=190, height=178, width=104)
        lab42_lis46.configure(background="white")
        lab42_lis46.configure(disabledforeground="#a3a3a3")
        lab42_lis46.configure(font=font10)
        lab42_lis46.configure(foreground="#000000")
        lab42_lis46.configure(width=104)
        lab42_lis46.insert(1, "GRNN")
        lab42_lis46.insert(2, "OLS")
        lab42_lis46.insert(3, "PLS")
        lab42_lis46.insert(4, "PNR")
        lab42_lis46.insert(5, "MLPR")
        lab42_lis46.bind('<<ListboxSelect>>', self.Curselect2)

        labelModels1 = Label(lab42)
        labelModels1.place(relx=0.22, rely=0.28, height=46, width=105)
        labelModels1.configure(activebackground="#808080")
        labelModels1.configure(background="#c0c0c0")
        labelModels1.configure(disabledforeground="#a3a3a3")
        labelModels1.configure(font=font13)
        labelModels1.configure(foreground="#000000")
        labelModels1.configure(text='''Models''')
        labelModels1.configure(width=105)

        lab43 = LabelFrame(self)
        lab43.place(x=240, y=20, height=395, width=180)
        lab43.configure(relief=RAISED)
        lab43.configure(font=font12)
        lab43.configure(foreground="black")
        lab43.configure(relief=RAISED)
        lab43.configure(text='''Classification''')
        lab43.configure(background="#c0c0c0")
        lab43.configure(highlightbackground="#f0f0f0f0f0f0")
        lab43.configure(highlightcolor="#c0c0c0")
        lab43.configure(width=180)

        lab43_lis47 = Listbox(lab43, exportselection=0)
        lab43_lis47.place(x=40, y=190, height=178, width=104)
        lab43_lis47.configure(background="white")
        lab43_lis47.configure(disabledforeground="#a3a3a3")
        lab43_lis47.configure(font=font10)
        lab43_lis47.configure(foreground="#000000")
        lab43_lis47.configure(width=104)
        lab43_lis47.insert(1, "KNN")
        lab43_lis47.insert(2, "LDA")
        lab43_lis47.insert(2, "SVM")
        lab43_lis47.bind('<<ListboxSelect>>', self.Curselect1)
        lab43_lis47.configure()

        lab44 = LabelFrame(self)
        lab44.place(x=450, y=20, height=395, width=180)
        lab44.configure(relief=RAISED)
        lab44.configure(font=font12)
        lab44.configure(foreground="black")
        lab44.configure(relief=RAISED)
        lab44.configure(text='''Properties''')
        lab44.configure(background="#c0c0c0")
        lab44.configure(highlightbackground="#f0f0f0f0f0f0")
        lab44.configure(highlightcolor="#c0c0c0")
        lab44.configure(width=180)

        lab42_lis46 = Listbox(lab44, exportselection=0)
        lab42_lis46.place(x=38, y=190, height=178, width=107)
        lab42_lis46.configure(background="white")
        lab42_lis46.configure(disabledforeground="#a3a3a3")
        lab42_lis46.configure(font=font10)
        lab42_lis46.configure(foreground="#000000")
        lab42_lis46.configure(width=104)
        lab42_lis46.insert(1, "Cetane Number")
        lab42_lis46.insert(2, "Flash Point")
        lab42_lis46.bind('<<ListboxSelect>>', self.Curselect3)

        labelModels2 = Label(lab43)
        labelModels2.place(relx=0.22, rely=0.28, height=46, width=105)
        labelModels2.configure(background="#c0c0c0")
        labelModels2.configure(disabledforeground="#a3a3a3")
        labelModels2.configure(font=font13)
        labelModels2.configure(foreground="#000000")
        labelModels2.configure(text='''Models''')
        labelModels2.configure(width=105)

        btnbegin = Button(self)
        btnbegin.place(relx=0.77, rely=0.6, height=63, width=106)
        btnbegin.configure(activebackground="#d9d9d9")
        btnbegin.configure(activeforeground="#000000")
        btnbegin.configure(background="#d9d9d9")
        btnbegin.configure(disabledforeground="#a3a3a3")
        btnbegin.configure(foreground="#000000")
        btnbegin.configure(highlightbackground="#d9d9d9")
        btnbegin.configure(highlightcolor="black")
        btnbegin.configure(pady="0")
        btnbegin.configure(text='''Begin''')
        btnbegin.configure(command = lambda: controller.show_frame(Graphs))
        btnbegin.configure(width=106)

        btnback = Button(self)
        btnback.place(relx=0.77, rely=0.78, height=63, width=106)
        btnback.configure(activebackground="#d9d9d9")
        btnback.configure(activeforeground="#000000")
        btnback.configure(background="#d9d9d9")
        btnback.configure(disabledforeground="#a3a3a3")
        btnback.configure(foreground="#000000")
        btnback.configure(highlightbackground="#d9d9d9")
        btnback.configure(highlightcolor="black")
        btnback.configure(pady="0")
        btnback.configure(text='''Back''')
        btnback.configure(command=lambda: controller.show_frame(Model_Selection))
        btnback.configure(width=106)

    def Curselect3(self, event): #Properties
        global prop
        widget = event.widget
        select = widget.curselection()
        value = widget.get(select[0])
        prop = value
        print(prop)

    def Curselect1(self, event): #Classification
        global cl
        widget = event.widget
        select = widget.curselection()
        value1 = widget.get(select[0])
        cl = value1
        print(cl)

    def Curselect2(self, event): #Linear Regression
        global nr
        widget = event.widget
        select = widget.curselection()
        value2 = widget.get(select[0])
        nr = value2
        print(nr)

class Graphs(Models):

    def calc(self, CID):
        X = descriptor_generator(CID)
        return X

    def family(self, X, cl):
        if cl == "KNN":
            fam = predict_family_knn(X)[0]
        elif cl == "LDA":
            fam = predict_family_lda(X)[0]
        elif cl == "SVM":
            fam = predict_family_svm(X)[0]
        else:
            tkMessageBox.askretrycancel("Class: ","You have not chosen proper class yet")
        return fam

    def fig_class(self, fam, cl):
        if cl == "KNN":
            fig1 = plot_knn(fam)
        elif cl == "LDA":
            fig1 = plot_lda(fam)
        elif cl == "SVM":
            fig1 = plot_svm(fam)
        else:
            tkMessageBox.askretrycancel("Class: ","You have not chosen proper class yet")
        return fig1

    def fig_nr(self,fam, nr, prop):
        if nr=="GRNN":
            fig2=GRNN_test(fam,prop)
        elif nr=="MLPR":
            fig2=MLPR_test(fam,prop)
        elif nr=="OLS":
            fig2=OLS_test(fam,prop)
        elif nr=="PLS":
            fig2=PLS_test(fam,prop)
        elif nr=="PNR":
            fig2=PNR_test(fam,prop)
        else:
            tkMessageBox.askretrycancel("Regression: ","You have not chosen proper method yet")
        return fig2

    def pred_result(self,fam,nr,prop,X):
        if nr == "GRNN":
            pred = GRNN_pred(fam, prop,X)
        elif nr == "MLPR":
            pred = MLPR_pred(fam, prop,X)
        elif nr == "OLS":
            pred = OLS_pred(fam, prop,X)
        elif nr == "PLS":
            pred = PLS_pred(fam, prop,X)
        elif nr == "PNR":
            pred = PNR_pred(fam, prop,X)
        else:
            tkMessageBox.askretrycancel("Regression: ", "You have not chosen proper method yet")
        return pred

    def result(self, CID, cl, nr, prop):
        X = self.calc(CID)
        fam = self.family(X, cl)
        f = Figure(figsize=(10, 4), dpi=100)
        pred=self.pred_result(fam,nr,prop,X)

        label1=Label(self,text="The predicted family is %s" % fam)
        label1.pack()

        label2=Label(self,text="The predicted value is %d" % pred)
        label2.pack()

        fig1 = self.fig_class(fam, cl)
        fig2 = self.fig_nr(fam, nr, prop)
        canvas = FigureCanvasTkAgg(fig1)
        canvas2 = FigureCanvasTkAgg(fig2)

        canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=True)
        canvas2.get_tk_widget().pack(side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        toolbar = NavigationToolbar2TkAgg(self, canvas)
        toolbar.update()
        canvas._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=True)

    def __init__(self, parent, controller):

        tk.Frame.__init__(self, parent)
        label = tk.Label(self, text="Graph Page")
        label.pack(pady=10, padx=10)
        button1 = tk.Button(self, text="result", command=lambda: self.result(CID, cl, nr, prop))
        button1.pack()

  #      f = Figure(figsize=(10, 4), dpi=100)
  #      a = f.add_subplot(111)
   #     a.plot([1,2.3,4,5],[1,2.3,4,6])
   #     canvas=FigureCanvasTkAgg()








app=QSPR_Modeling()
app.mainloop()


from setuptools import setup, find_packages

setup(
    name='Biofuel_MyProject',
    version='0.1.0',
    description='Predict flash point and cetane number of biofuel',
    long_description=open('README.md').read(),
    url='https://github.com/Zhangjt9317/Biofuel-Group-Project',
    author='Jingtian Zhang, Cheng Zeng, Renlong Zheng, Chenggang Xi',
    author_email='jtz9317@gmail.com',
    license='MIT',
    keywords='flash point and cetane number',
    packages=find_packages(exclude=['tests']),
    install_requires=['neupy', 'pubchempy', 'scikit-learn'],
    project_urls={
        'Bug Reports': 'https://github.com/Zhangjt9317/Biofuel-Group-Project/issues',
        'Source': 'https://github.com/Zhangjt9317/Biofuel-Group-Project',
    },

)